uses csvconfigurable,varsubstunit;

var version : String;

const
  DEFAULT_POINT_TEMPLATE = '%ID%;%REPLACE(%NAME%;;;,)%;%CORRECTEDCOORDINATE%;%REPLACE(%DIFFICULTY%;.;,)%;%REPLACE(%TERRAIN%;.;,)%;%TYPE%;%SIZE%';
  DEFAULT_WAYPOINT_TEMPLATE = '%WPTID%;%REPLACE(%WPTNAME%;;;,)%;%WPTCOORDINATE%;%REPLACE(%DIFFICULTY%;.;,)%;%REPLACE(%TERRAIN%;.;,)%;%WPTTYPE%;%SIZE%';

function ExportExtension: string;
begin
  result := 'csv';
end;

function ExportHint: string;
begin
  result := _('Exports the list into text file with comma-separated values. The exported fields can be customized with VarSubst variables.');
end;

function ExportDescription: string;
begin
  result := _('CSV Configurable');
end;

function GetSettingsFile: string;
begin
   Result := ReplaceString(GEOGET_SCRIPTFULLNAME,'.gge.pas','.ini');
end;

function GetPackageMeta: string;
begin
   Result := ReplaceString(GEOGET_SCRIPTFULLNAME,'.gge.pas','.meta.ini');
end;

function CleanupToLine(line : string): String;
begin
   Result := line;
   Result := RegexReplace('(\r\n)+',Result,' ',false);
   Result := RegexReplace('(\n)+',Result,' ',false);
   Result := RegexReplace('(\r)+',Result,' ',false);
end;

procedure CSVConfigurableExportButtonRefresh;
begin
   CSVConfigurable_ExportButton.Enabled := CSVConfigurable_PointCheckBox.Checked or CSVConfigurable_WaypointCheckBox.Checked;
end;

procedure VarsubstEdit(edit : TMemo);
var value : String;
begin
  value := VarSubstDlgTemplateStringNoSave(_('VarSubst pattern editor'),VarSubstRandomGeoID,AnsiToUtf(edit.Text));
  if value <> '' then edit.Text := value;
end;

procedure CSVConfigurablePointEditButtonOnClick(Sender : TObject);
begin
   VarsubstEdit(CSVConfigurable_PointMemo);
end;

procedure CSVConfigurableWaypointEditButtonOnClick(Sender : TObject);
begin
   VarsubstEdit(CSVConfigurable_WaypointMemo);
end;

procedure CSVConfigurablePointCheckBoxOnClick(Sender :TObject);
begin
   CSVConfigurable_PointMemo.Enabled := CSVConfigurable_PointCheckBox.Checked;
   CSVConfigurable_PointEditButton.Enabled := CSVConfigurable_PointCheckBox.Checked;
   CSVConfigurableExportButtonRefresh;
end;

procedure CSVConfigurableWaypointCheckBoxOnClick(Sender :TObject);
begin
   CSVConfigurable_WaypointMemo.Enabled := CSVConfigurable_WaypointCheckBox.Checked;
   CSVConfigurable_WaypointEditButton.Enabled := CSVConfigurable_WaypointCheckBox.Checked;
   CSVConfigurableExportButtonRefresh;
end;

procedure LoadSettings;
var ini : TIniFile;
begin
   ini := TiniFile.Create(GetSettingsFile);
   try
     CSVConfigurable_PointMemo.Text := HtmlEntityDecode(ini.ReadString('LastValues','PointMemo',DEFAULT_POINT_TEMPLATE));
     CSVConfigurable_WaypointMemo.Text := HtmlEntityDecode(ini.ReadString('LastValues','WaypointMemo',DEFAULT_WAYPOINT_TEMPLATE));
     CSVConfigurable_PointCheckBox.Checked := ini.ReadBool('LastValues','PointExport',true);
     CSVConfigurable_WaypointCheckBox.Checked := ini.ReadBool('LastValues','WaypointExport',true);
     
     CSVConfigurablePointCheckBoxOnClick(nil);
     CSVConfigurableWaypointCheckBoxOnClick(nil);
   finally
     ini.Free;
   end;
end;

procedure SaveSettings;
var ini : TIniFile;
begin
   ini := TiniFile.Create(GetSettingsFile);
   try
     ini.WriteString('LastValues','PointMemo',HtmlEntityEncode(CleanupToLine(CSVConfigurable_PointMemo.Text)));
     ini.WriteString('LastValues','WaypointMemo',HtmlEntityEncode(CleanupToLine(CSVConfigurable_WaypointMemo.Text)));
     ini.WriteBool('LastValues','PointExport',CSVConfigurable_PointCheckBox.Checked);
     ini.WriteBool('LastValues','WaypointExport',CSVConfigurable_WaypointCheckBox.Checked);
   finally
     ini.Free;
   end;
end;

procedure CSVConcifugrableVarSubstLabelOnClick(Sender : TObject);
begin
    RunShell('http://geoget.ararat.cz/doku.php/user:skript:varsubst');
end;

procedure Init;
begin
    //Inicializovat hodnoty dialogu
    version := GeoGetPackageVersion(GetPackageMeta());
    CSVConfigurable.Caption := CSVConfigurable.Caption + ' (' + version + ')';
    
    CSVConfigurable_PointCheckBox.OnClick := @CSVConfigurablePointCheckBoxOnClick;
    CSVConfigurable_WaypointCheckBox.OnClick := @CSVConfigurableWaypointCheckBoxOnClick;
    CSVConfigurable_PointEditButton.OnClick := @CSVConfigurablePointEditButtonOnClick;
    CSVConfigurable_WaypointEditButton.OnClick := @CSVConfigurableWaypointEditButtonOnClick;
    CSVConfigurable_VarSubstLabel.OnClick := @CSVConcifugrableVarSubstLabelOnClick;
end;

procedure Cleanup;
begin
    CSVConfigurable.Release;
end;

function ShowDialog : boolean;
var i : Integer;
begin
   result := false;
   LoadSettings;
   i := CSVConfigurable.showmodal;
   if i = 1 then begin
      SaveSettings;
      result := true;
   end else GeoStopWork;
end;

procedure ExportStart;
begin
   Init;
   ShowDialog;
end;

function ExportHeader: string;
begin
   Result := '';
end;

function ExportPoint: string;
var vsres : boolean;
    s : string;
    n : Integer;
begin
    vsres := false;
    s := '';
    Result := '';
    
   if CSVConfigurable_PointCheckBox.Checked then
   begin
     vsres := VarSubstGeo(GC,AnsiToUtf(CSVConfigurable_PointMemo.Text),VARSUBST_ANSI,s);
     if not vsres then begin
       GeoStopWork;
       ShowMessage(_('Problem in parsing VarSubst template on point') + ' ' + GC.ID);
     end else Result := CleanupToLine(s) + CRLF;      
   end;
  
   if CSVConfigurable_WaypointCheckBox.Checked then
   begin
      for n := 0 to GC.Waypoints.Count-1 do
      begin
        vsres := VarSubstWpt(GC.Waypoints[n],AnsiToUtf(CSVConfigurable_WaypointMemo.Text),VARSUBST_ANSI,s);
        if not vsres then begin
          GeoStopWork;
          ShowMessage(_('Problem in parsing VarSubst template on waypoint') + ' ' + GC.Waypoints[n].ID);
        end else Result := Result + CleanupToLine(s) + CRLF; 
      end;   
   end;
end;

function ExportAfter(value: string): string;
begin
  Result := ''; 
  if GeoBusyTest then DeleteFile(value);
end;

procedure ExportStop;
begin
  Cleanup;
end;
